#include "DebugInterface.h"
#include "OgreOverlayContainer.h"
#include "OgreOverlayElement.h"
#include "OgreOverlay.h"
#include "OgreOverlayManager.h"
#include "OgreTextAreaOverlayElement.h"
#include "OgreResourceManager.h"
#include "OgreFontManager.h"
#include "LuaWrapper.h"

DebugInterface::DebugInterface(void)
{
    // There is a bug in current version of Ogre, doesn't seem to load fonts like it should.  This will
    // kick the resouce manager so that the fonts load properly (really shouldn't need it, but ...)
	Ogre::ResourceManager::ResourceMapIterator iter = Ogre::FontManager::getSingleton().getResourceIterator();
	while (iter.hasMoreElements()) { iter.getNext()->load(); }

	Ogre::OverlayManager *olm;
	Ogre::OverlayContainer *panel ;
    Ogre::String szElement;

	olm=Ogre::OverlayManager::getSingletonPtr();

	mDebugOverlay=olm->create("DebuggingOverlay");  
	// Next, we create a panel within this overlay:
    panel=static_cast<Ogre::OverlayContainer*>(olm->createOverlayElement("Panel","TextPanel"));
    panel->setMetricsMode(Ogre::GMM_RELATIVE); 
    panel->setPosition(.1f,.1f);
    panel->setDimensions(0.5f,0.5f);         
    mDebugOverlay->add2D(panel);    

    mOverlayText=static_cast<Ogre::TextAreaOverlayElement*>(olm->createOverlayElement("TextArea","DebugOutput"));

    mOverlayText->setFontName("BlueHighwayBig"); 
    mOverlayText->setMetricsMode(Ogre::GMM_PIXELS);   
    mOverlayText->setCaption("Lua >");	              // Set what we want the text area to show
    
    mOverlayText->setCharHeight(23);	              // Set the height of the text (in pixels)
    
    // Add the text area to the panel
    panel->addChild(mOverlayText);
    
	// Start out inactive
    mDebugOverlay->hide();
    mIsActive = false;

}



void 
DebugInterface::HandleKeystroke(OIS::KeyEvent e)
{
        if (e.key == OIS::KC_RETURN)
        {
            LuaWrapper::getSingleton()->doString(mLuaCommand.c_str());
            mHistory.push_back(mLuaCommand);
            mHistoryIndex = (int) mHistory.size();
            mLuaCommand = "";
            mOverlayText->setCaption("Lua >" + mLuaCommand);
        }
        else if (e.key == OIS::KC_BACK || e.key == OIS::KC_LEFT)
        {
            if (mLuaCommand.size() > 0)
            {
                mLuaCommand.erase(mLuaCommand.size()-1);
                mOverlayText->setCaption("Lua >" + mLuaCommand);
            }
        }
        else if (e.key == OIS::KC_UP)
        {
            mHistoryIndex--;
            mHistoryIndex = std::max<int>(mHistoryIndex,0);
            if (mHistoryIndex < mHistory.size())
            {
                mLuaCommand = mHistory[mHistoryIndex];
                mOverlayText->setCaption("Lua >" + mLuaCommand);
            }
        }
        else if (e.key == OIS::KC_DOWN)
        {
            mHistoryIndex++;
            mHistoryIndex = (int) std::min<size_t>(mHistoryIndex,mHistory.size()-1);
            if (mHistoryIndex < mHistory.size())
            {
                mLuaCommand = mHistory[mHistoryIndex];
                mOverlayText->setCaption("Lua >" + mLuaCommand);
            }
        }

        else if (e.key != OIS::KC_LSHIFT && e.key != OIS::KC_RSHIFT && e.key != OIS::KC_LCONTROL && e.key != OIS::KC_RCONTROL)
        {
            mLuaCommand.push_back((char) e.text);
            mOverlayText->setCaption("Lua >" + mLuaCommand);
        }
}


void 
DebugInterface::SetDebugConsoleActive(bool activate)
{
    if (activate != mIsActive)
    {
        mIsActive = activate;
        if (mIsActive)
        {
            mLuaCommand = "";
            mOverlayText->setCaption("Lua >");
            mDebugOverlay->show();
        }
        else
        {
            mDebugOverlay->hide();
        }
    }
}


DebugInterface::~DebugInterface(void)
{
}
