#include "InputHandler.h"
#include "World.h"
#include "OgreRenderWindow.h"
#include "OgreStringConverter.h"
#include "DebugInterface.h"
#include <OIS/OIS.h>

#include <stdio.h>

InputHandler::InputHandler(Ogre::RenderWindow *renderWindow,DebugInterface *dbInterface) : 
	 mRenderWindow(renderWindow), mSpaceDown(false) , mDebugInterface(dbInterface)
{
	OIS::ParamList pl;
	size_t windowHnd = 0;
	std::ostringstream windowHndStr;

	renderWindow->getCustomAttribute("WINDOW", &windowHnd);
	windowHndStr << windowHnd;
	pl.insert(std::make_pair(std::string("WINDOW"), windowHndStr.str()));

	mInputManager = OIS::InputManager::createInputSystem( pl );

	mKeyboard = static_cast<OIS::Keyboard*>(mInputManager->createInputObject( OIS::OISKeyboard, true /* buffered */ ));
    mKeyboard->setEventCallback(this);
}


 bool 
 InputHandler::keyPressed(const OIS::KeyEvent &e)
 {
    if (e.key == OIS::KC_GRAVE)
    {
        mDebugInterface->SetDebugConsoleActive(!mDebugInterface->DebugConsoleActive());
    }
    else  if (mDebugInterface->DebugConsoleActive())
    {
        mDebugInterface->HandleKeystroke(e);
    }
    return true;
 }
   
 bool 
 InputHandler::keyReleased(const OIS::KeyEvent &e)
 {
     return false;
 }


 bool 
 InputHandler::isKeyDown(const OIS::KeyCode key)
 {
     return mKeyboard->isKeyDown(key);
 }



void 
InputHandler::Think(float time)
{
    // Capture input for this frame
	mKeyboard->capture();
}

