#include "LuaWrapper.h"
#include "World.h"

extern "C"
{
	#include "lua.h"
	#include "lualib.h"
	#include "lauxlib.h"

}

//////////////////////////////////////
// Code for making the singleton work
//////////////////////////////////////

LuaWrapper *LuaWrapper::_instance = 0;

LuaWrapper* 
LuaWrapper::getSingleton()
{
    if (!_instance)
    {
        _instance = new LuaWrapper();
    }
    return _instance;
}


//////////////////////////////////////////////
/// C/C++ functions we are exposing to lua
///////////////////////////////////////////////

int createObject(lua_State *L)
{
    int numArgs = lua_gettop(L);

    if (numArgs != 2)
    {
        return 0;
    }
    const char* meshName = lua_tostring(L,1);
    const char* nodeName = lua_tostring(L,2);
    LuaWrapper::getSingleton()->getWorld()->createObject(meshName, nodeName);
    return 0;
}


int setPosition(lua_State *L)
{
    int numArgs = lua_gettop(L);

    if (numArgs < 4)
    {
        return 0;
    }
    const char* nodeName = lua_tostring(L,1);
    float x = (float) lua_tonumber(L,2);
    float y = (float) lua_tonumber(L,3);
    float z = (float) lua_tonumber(L,4);
    LuaWrapper::getSingleton()->getWorld()->setPosition(nodeName, Ogre::Vector3(x,y,z));
    return 0;
}


////////////////////////////////////////////////
///  Registering functions with lua
//////////////////////////////////////////////////

void LuaWrapper::initializeLuaFunctions()
{
    lua_register(mLuaState, "createObject", createObject);
    lua_register(mLuaState, "setPosition", setPosition);

}



void 
LuaWrapper::setWorld(World *world)
{
    mWorld = world;
}


void  LuaWrapper::doString(const char *s)
{ 
    luaL_dostring(mLuaState, s); 
}


LuaWrapper::LuaWrapper()
{
    mLuaState = luaL_newstate();
    luaL_openlibs(mLuaState);
    initializeLuaFunctions();
}

LuaWrapper::~LuaWrapper()
{
    lua_close(mLuaState);
}
